@@ -10,7 +10,7 @@ use Carp qw(confess);
 
 use namespace::clean;
 
-our $VERSION = '2.03';
+our $VERSION = '2.04';
 
 use overload
 	q{""} => "as_string",
@@ -67,7 +67,7 @@ sub as_string
 
 {
 
-	package Form::Tiny::Error::DoesNotExist;
+	package Form::Tiny::Error::Required;
 	use parent -norequire, "Form::Tiny::Error";
 
 	sub default_error
@@ -76,12 +76,6 @@ sub as_string
 	}
 }
 
-{
-
-	package Form::Tiny::Error::Required;
-	use parent -norequire, "Form::Tiny::Error::DoesNotExist";
-}
-
 {
 
 	package Form::Tiny::Error::IsntStrict;
@@ -142,8 +136,6 @@ A couple of in-place subclasses are provided to differentiate the type of error
 
 =item * Form::Tiny::Error::Required
 
-=item * Form::Tiny::Error::DoesNotExist (DEPRECATED, use Form::Tiny::Error::Required)
-
 =item * Form::Tiny::Error::IsntStrict
 
 =item * Form::Tiny::Error::DoesNotValidate
