@@ -11,15 +11,23 @@ use Carp qw(croak carp);
 use Form::Tiny::FieldDefinitionBuilder;
 use Form::Tiny::Hook;
 use Form::Tiny::Error;
-use Form::Tiny::Utils qw(try);
+use Form::Tiny::Utils qw(try uniq);
+require Moo::Role;
 
 use namespace::clean;
 
-our $VERSION = '2.03';
+our $VERSION = '2.04';
 
 # more clear error messages in some crucial cases
 our @CARP_NOT = qw(Form::Tiny Form::Tiny::Form);
 
+has 'package' => (
+	is => 'ro',
+	writer => 'set_package',
+	isa => Str,
+	predicate => 1,
+);
+
 has 'fields' => (
 	is => 'ro',
 	writer => 'set_fields',
@@ -45,14 +53,26 @@ has 'complete' => (
 	default => sub { 0 },
 );
 
+has 'meta_roles' => (
+	is => 'ro',
+	writer => 'set_meta_roles',
+	isa => ArrayRef,
+	default => sub { [] },
+);
+
+has 'form_roles' => (
+	is => 'ro',
+	writer => 'set_form_roles',
+	isa => ArrayRef,
+	default => sub { [] },
+);
+
 has 'messages' => (
 	is => 'ro',
 	isa => HashRef [Str],
 	default => sub { {} },
 );
 
-sub consistent_api { 0 }
-
 sub build_error
 {
 	my ($self, $name, %params) = @_;
@@ -157,7 +177,7 @@ sub add_message
 
 	my $isa;
 	my $err = try sub {
-		$isa = "Form::Tiny::Error::$name"->isa('Form::Tiny::Error')
+		$isa = "Form::Tiny::Error::$name"->isa('Form::Tiny::Error');
 	};
 
 	croak "$name is not a valid Form::Tiny error class name"
@@ -167,6 +187,27 @@ sub add_message
 	return $self;
 }
 
+# this is required so that proper hooks on inherit_from can be fired
+sub inherit_roles_from
+{
+	my ($self, $parent) = @_;
+
+	if (defined $parent) {
+		$self->set_meta_roles([uniq(@{$parent->meta_roles}, @{$self->meta_roles})]);
+		$self->set_form_roles([uniq(@{$parent->form_roles}, @{$self->form_roles})]);
+	}
+
+	Moo::Role->apply_roles_to_object(
+		$self, @{$self->meta_roles}
+	) if @{$self->meta_roles};
+
+	Moo::Role->apply_roles_to_package(
+		$self->package, @{$self->form_roles}
+	) if $self->has_package && @{$self->form_roles};
+
+	return $self;
+}
+
 sub inherit_from
 {
 	my ($self, $parent) = @_;
